import {
    maxValueByUnit,
    orderedTimeUnits,
    spendDirectives,
} from "commitlint-plugin-spend/src/constants";

module.exports = (ctx) => {
  const { body } = ctx.payload;

  if (!body) {
    // Require a body
    return [
      false,
      "Commit message must contain a body with the spend directive",
    ];
  }

  const lines = body.split("\n");

  const spendLines = lines.filter((line) =>
    spendDirectives.some((directive) => line.startsWith(directive))
  );

  const isValid = spendLines.every((line) => {
    const timeValues = line.split(" ").slice(1);
    return validateTimeUnit(timeValues);
  });

  return [
    isValid,
    isValid
      ? "Valid spend directive"
      : "Commit message must contain a valid GitLab spend directive (e.g., /spend 3h)",
  ];
};

/**
 * Validate a given string array based on the GitLab time unit pattern.
 * @see https://docs.gitlab.com/ee/user/project/time_tracking.html#available-time-units
 *
 * @param timeValues - The time values to validate sorted from smallest to largest time unit (e.g., ["3m", "2h", "1d"])
 * @returns - True if the time values are valid and in the correct order, false otherwise
 */
function validateTimeUnit(timeValues) {
  let lastValueIndex = -1;
  for (const unit of orderedTimeUnits) {
    const [short, singular, plural] = unit;

    const timeValueIndex = timeValues.findIndex(
      (value) =>
        value.endsWith(short) ||
        value.endsWith(singular) ||
        value.endsWith(plural)
    );

    const timeValue = timeValues[timeValueIndex];

    if (timeValueIndex !== -1) {
      if (timeValueIndex <= lastValueIndex) {
        // The time values are not in the correct order
        return false;
      }
      // Remove the time unit from the time value
      const timeValueNumber = timeValue
        .replace(short, "")
        .replace(singular, "")
        .replace(plural, "");

      const timeValueNumberAsNumber = parseInt(timeValueNumber, 10);

      if (isNaN(timeValueNumberAsNumber)) {
        return false;
      }

      if (timeValueNumberAsNumber > maxValueByUnit[short]) {
        return false;
      }

      // The time values are in the correct order
      lastValueIndex = timeValueIndex;
    }
  }

  return lastValueIndex === -1;
}
